@@ -11,36 +11,18 @@
 #endif
 
 #include <ctype.h>
-#include <string.h>
-#include <glib.h>
-
-#ifdef USE_NTLM_AUTH
-#include <stdlib.h>
 #include <errno.h>
-#endif
+#include <stdlib.h>
+#include <string.h>
+
 #include "soup-auth-manager-ntlm.h"
+#include "soup.h"
 #include "soup-auth-ntlm.h"
-#include "soup-message.h"
 #include "soup-message-private.h"
-#include "soup-misc.h"
-#include "soup-session.h"
-#include "soup-session-feature.h"
-#include "soup-uri.h"
 
 static void soup_auth_manager_ntlm_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
 static SoupSessionFeatureInterface *soup_auth_manager_parent_feature_interface;
 
-static void attach (SoupSessionFeature *feature, SoupSession *session);
-static void request_queued (SoupSessionFeature *feature, SoupSession *session,
-			    SoupMessage *msg);
-static void request_started (SoupSessionFeature *feature, SoupSession *session,
-			     SoupMessage *msg, SoupSocket *socket);
-static void request_unqueued (SoupSessionFeature *feature,
-			      SoupSession *session, SoupMessage *msg);
-static gboolean add_feature (SoupSessionFeature *feature, GType type);
-static gboolean remove_feature (SoupSessionFeature *feature, GType type);
-static gboolean has_feature (SoupSessionFeature *feature, GType type);
-
 G_DEFINE_TYPE_WITH_CODE (SoupAuthManagerNTLM, soup_auth_manager_ntlm, SOUP_TYPE_AUTH_MANAGER,
 			 G_IMPLEMENT_INTERFACE (SOUP_TYPE_SESSION_FEATURE,
 						soup_auth_manager_ntlm_session_feature_init))
@@ -74,6 +56,8 @@ typedef struct {
 #endif
 } SoupNTLMConnection;
 
+static void free_ntlm_connection (SoupNTLMConnection *conn);
+
 typedef struct {
 	gboolean use_ntlm;
 
@@ -105,7 +89,8 @@ soup_auth_manager_ntlm_init (SoupAuthManagerNTLM *ntlm)
 	SoupAuthManagerNTLMPrivate *priv =
 		SOUP_AUTH_MANAGER_NTLM_GET_PRIVATE (ntlm);
 
-	priv->connections_by_id = g_hash_table_new (NULL, NULL);
+	priv->connections_by_id = g_hash_table_new_full (NULL, NULL, NULL,
+							 (GDestroyNotify)free_ntlm_connection);
 	priv->connections_by_msg = g_hash_table_new (NULL, NULL);
 #ifdef USE_NTLM_AUTH
 	priv->ntlm_auth_accessible = (access (NTLM_AUTH, X_OK) == 0);
@@ -118,8 +103,7 @@ free_ntlm_connection (SoupNTLMConnection *conn)
 	g_free (conn->response_header);
 	g_free (conn->nonce);
 	g_free (conn->domain);
-	if (conn->auth)
-		g_object_unref (conn->auth);
+	g_clear_object (&conn->auth);
 #ifdef USE_NTLM_AUTH
 	g_free (conn->challenge_header);
 	sso_ntlm_close (conn);
@@ -128,19 +112,11 @@ free_ntlm_connection (SoupNTLMConnection *conn)
 }
 
 static void
-free_ntlm_connection_foreach (gpointer key, gpointer value, gpointer user_data)
-{
-	free_ntlm_connection (value);
-}
-
-static void
-finalize (GObject *object)
+soup_auth_manager_ntlm_finalize (GObject *object)
 {
 	SoupAuthManagerNTLMPrivate *priv =
 		SOUP_AUTH_MANAGER_NTLM_GET_PRIVATE (object);
 
-	g_hash_table_foreach (priv->connections_by_id,
-			      free_ntlm_connection_foreach, NULL);
 	g_hash_table_destroy (priv->connections_by_id);
 	g_hash_table_destroy (priv->connections_by_msg);
 
@@ -154,27 +130,11 @@ soup_auth_manager_ntlm_class_init (SoupAuthManagerNTLMClass *auth_manager_ntlm_c
 
 	g_type_class_add_private (auth_manager_ntlm_class, sizeof (SoupAuthManagerNTLMPrivate));
 
-	object_class->finalize = finalize;
+	object_class->finalize = soup_auth_manager_ntlm_finalize;
 }
 
 static void
-soup_auth_manager_ntlm_session_feature_init (SoupSessionFeatureInterface *feature_interface,
-					     gpointer interface_data)
-{
-	soup_auth_manager_parent_feature_interface =
-		g_type_interface_peek_parent (feature_interface);
-
-	feature_interface->attach = attach;
-	feature_interface->request_queued = request_queued;
-	feature_interface->request_started = request_started;
-	feature_interface->request_unqueued = request_unqueued;
-	feature_interface->add_feature = add_feature;
-	feature_interface->remove_feature = remove_feature;
-	feature_interface->has_feature = has_feature;
-}
-
-static void
-attach (SoupSessionFeature *manager, SoupSession *session)
+soup_auth_manager_ntlm_attach (SoupSessionFeature *manager, SoupSession *session)
 {
 	SoupAuthManagerNTLMPrivate *priv =
 		SOUP_AUTH_MANAGER_NTLM_GET_PRIVATE (manager);
@@ -189,11 +149,7 @@ static void
 delete_conn (SoupSocket *socket, gpointer user_data)
 {
 	SoupAuthManagerNTLMPrivate *priv = user_data;
-	SoupNTLMConnection *conn;
 
-	conn = g_hash_table_lookup (priv->connections_by_id, socket);
-	if (conn)
-		free_ntlm_connection (conn);
 	g_hash_table_remove (priv->connections_by_id, socket);
 	g_signal_handlers_disconnect_by_func (socket, delete_conn, priv);
 }
@@ -525,7 +481,9 @@ done:
 }
 
 static void
-request_queued (SoupSessionFeature *ntlm, SoupSession *session, SoupMessage *msg)
+soup_auth_manager_ntlm_request_queued (SoupSessionFeature *ntlm,
+				       SoupSession *session,
+				       SoupMessage *msg)
 {
 	SoupAuthManagerNTLMPrivate *priv =
 		SOUP_AUTH_MANAGER_NTLM_GET_PRIVATE (ntlm);
@@ -543,8 +501,10 @@ request_queued (SoupSessionFeature *ntlm, SoupSession *session, SoupMessage *msg
 }
 
 static void
-request_started (SoupSessionFeature *ntlm, SoupSession *session,
-		 SoupMessage *msg, SoupSocket *socket)
+soup_auth_manager_ntlm_request_started (SoupSessionFeature *ntlm,
+					SoupSession *session,
+					SoupMessage *msg,
+					SoupSocket *socket)
 {
 	SoupAuthManagerNTLMPrivate *priv =
 		SOUP_AUTH_MANAGER_NTLM_GET_PRIVATE (ntlm);
@@ -626,8 +586,9 @@ super:
 }
 
 static void
-request_unqueued (SoupSessionFeature *ntlm, SoupSession *session,
-		  SoupMessage *msg)
+soup_auth_manager_ntlm_request_unqueued (SoupSessionFeature *ntlm,
+					 SoupSession *session,
+					 SoupMessage *msg)
 {
 	g_signal_handlers_disconnect_by_func (msg, ntlm_authorize_pre, ntlm);
 	g_signal_handlers_disconnect_by_func (msg, ntlm_authorize_post, ntlm);
@@ -636,7 +597,7 @@ request_unqueued (SoupSessionFeature *ntlm, SoupSession *session,
 }
 
 static gboolean
-add_feature (SoupSessionFeature *feature, GType type)
+soup_auth_manager_ntlm_add_feature (SoupSessionFeature *feature, GType type)
 {
 	SoupAuthManagerNTLMPrivate *priv =
 		SOUP_AUTH_MANAGER_NTLM_GET_PRIVATE (feature);
@@ -650,7 +611,7 @@ add_feature (SoupSessionFeature *feature, GType type)
 }
 
 static gboolean
-remove_feature (SoupSessionFeature *feature, GType type)
+soup_auth_manager_ntlm_remove_feature (SoupSessionFeature *feature, GType type)
 {
 	SoupAuthManagerNTLMPrivate *priv =
 		SOUP_AUTH_MANAGER_NTLM_GET_PRIVATE (feature);
@@ -664,7 +625,7 @@ remove_feature (SoupSessionFeature *feature, GType type)
 }
 
 static gboolean
-has_feature (SoupSessionFeature *feature, GType type)
+soup_auth_manager_ntlm_has_feature (SoupSessionFeature *feature, GType type)
 {
 	SoupAuthManagerNTLMPrivate *priv =
 		SOUP_AUTH_MANAGER_NTLM_GET_PRIVATE (feature);
@@ -675,6 +636,22 @@ has_feature (SoupSessionFeature *feature, GType type)
 	return soup_auth_manager_parent_feature_interface->has_feature (feature, type);
 }
 
+static void
+soup_auth_manager_ntlm_session_feature_init (SoupSessionFeatureInterface *feature_interface,
+					     gpointer interface_data)
+{
+	soup_auth_manager_parent_feature_interface =
+		g_type_interface_peek_parent (feature_interface);
+
+	feature_interface->attach = soup_auth_manager_ntlm_attach;
+	feature_interface->request_queued = soup_auth_manager_ntlm_request_queued;
+	feature_interface->request_started = soup_auth_manager_ntlm_request_started;
+	feature_interface->request_unqueued = soup_auth_manager_ntlm_request_unqueued;
+	feature_interface->add_feature = soup_auth_manager_ntlm_add_feature;
+	feature_interface->remove_feature = soup_auth_manager_ntlm_remove_feature;
+	feature_interface->has_feature = soup_auth_manager_ntlm_has_feature;
+}
+
 /* NTLM code */
 
 static void md4sum                (const unsigned char *in, 
