@@ -12,31 +12,16 @@
 #include <string.h>
 
 #include "soup-auth-manager.h"
-#include "soup-address.h"
-#include "soup-headers.h"
+#include "soup.h"
 #include "soup-marshal.h"
 #include "soup-message-private.h"
 #include "soup-message-queue.h"
 #include "soup-path-map.h"
-#include "soup-session.h"
-#include "soup-session-feature.h"
 #include "soup-session-private.h"
-#include "soup-uri.h"
 
 static void soup_auth_manager_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
 static SoupSessionFeatureInterface *soup_session_feature_default_interface;
 
-static void attach (SoupSessionFeature *feature, SoupSession *session);
-static void request_queued  (SoupSessionFeature *feature, SoupSession *session,
-			     SoupMessage *msg);
-static void request_started  (SoupSessionFeature *feature, SoupSession *session,
-			      SoupMessage *msg, SoupSocket *socket);
-static void request_unqueued  (SoupSessionFeature *feature,
-			       SoupSession *session, SoupMessage *msg);
-static gboolean add_feature (SoupSessionFeature *feature, GType type);
-static gboolean remove_feature (SoupSessionFeature *feature, GType type);
-static gboolean has_feature (SoupSessionFeature *feature, GType type);
-
 enum {
 	AUTHENTICATE,
 	LAST_SIGNAL
@@ -63,47 +48,30 @@ typedef struct {
 	GHashTable  *auths;            /* scheme:realm -> SoupAuth */
 } SoupAuthHost;
 
+static void soup_auth_host_free (SoupAuthHost *host);
+
 static void
 soup_auth_manager_init (SoupAuthManager *manager)
 {
 	SoupAuthManagerPrivate *priv = SOUP_AUTH_MANAGER_GET_PRIVATE (manager);
 
-	priv->auth_types = g_ptr_array_new ();
-	priv->auth_hosts = g_hash_table_new (soup_uri_host_hash,
-					     soup_uri_host_equal);
-}
-
-static gboolean
-foreach_free_host (gpointer key, gpointer value, gpointer data)
-{
-	SoupAuthHost *host = value;
-
-	if (host->auth_realms)
-		soup_path_map_free (host->auth_realms);
-	if (host->auths)
-		g_hash_table_destroy (host->auths);
-
-	soup_uri_free (host->uri);
-	g_slice_free (SoupAuthHost, host);
-
-	return TRUE;
+	priv->auth_types = g_ptr_array_new_with_free_func ((GDestroyNotify)g_type_class_unref);
+	priv->auth_hosts = g_hash_table_new_full (soup_uri_host_hash,
+						  soup_uri_host_equal,
+						  NULL,
+						  (GDestroyNotify)soup_auth_host_free);
 }
 
 static void
-finalize (GObject *object)
+soup_auth_manager_finalize (GObject *object)
 {
 	SoupAuthManagerPrivate *priv = SOUP_AUTH_MANAGER_GET_PRIVATE (object);
-	int i;
 
-	for (i = 0; i < priv->auth_types->len; i++)
-		g_type_class_unref (priv->auth_types->pdata[i]);
 	g_ptr_array_free (priv->auth_types, TRUE);
 
-	g_hash_table_foreach_remove (priv->auth_hosts, foreach_free_host, NULL);
 	g_hash_table_destroy (priv->auth_hosts);
 
-	if (priv->proxy_auth)
-		g_object_unref (priv->proxy_auth);
+	g_clear_object (&priv->proxy_auth);
 
 	G_OBJECT_CLASS (soup_auth_manager_parent_class)->finalize (object);
 }
@@ -115,7 +83,7 @@ soup_auth_manager_class_init (SoupAuthManagerClass *auth_manager_class)
 
 	g_type_class_add_private (auth_manager_class, sizeof (SoupAuthManagerPrivate));
 
-	object_class->finalize = finalize;
+	object_class->finalize = soup_auth_manager_finalize;
 
 	signals[AUTHENTICATE] =
 		g_signal_new ("authenticate",
@@ -131,22 +99,6 @@ soup_auth_manager_class_init (SoupAuthManagerClass *auth_manager_class)
 
 }
 
-static void
-soup_auth_manager_session_feature_init (SoupSessionFeatureInterface *feature_interface,
-					gpointer interface_data)
-{
-	soup_session_feature_default_interface =
-		g_type_default_interface_peek (SOUP_TYPE_SESSION_FEATURE);
-
-	feature_interface->attach = attach;
-	feature_interface->request_queued = request_queued;
-	feature_interface->request_started = request_started;
-	feature_interface->request_unqueued = request_unqueued;
-	feature_interface->add_feature = add_feature;
-	feature_interface->remove_feature = remove_feature;
-	feature_interface->has_feature = has_feature;
-}
-
 static int
 auth_type_compare_func (gconstpointer a, gconstpointer b)
 {
@@ -157,7 +109,7 @@ auth_type_compare_func (gconstpointer a, gconstpointer b)
 }
 
 static gboolean
-add_feature (SoupSessionFeature *feature, GType type)
+soup_auth_manager_add_feature (SoupSessionFeature *feature, GType type)
 {
 	SoupAuthManagerPrivate *priv = SOUP_AUTH_MANAGER_GET_PRIVATE (feature);
 	SoupAuthClass *auth_class;
@@ -172,7 +124,7 @@ add_feature (SoupSessionFeature *feature, GType type)
 }
 
 static gboolean
-remove_feature (SoupSessionFeature *feature, GType type)
+soup_auth_manager_remove_feature (SoupSessionFeature *feature, GType type)
 {
 	SoupAuthManagerPrivate *priv = SOUP_AUTH_MANAGER_GET_PRIVATE (feature);
 	SoupAuthClass *auth_class;
@@ -185,7 +137,6 @@ remove_feature (SoupSessionFeature *feature, GType type)
 	for (i = 0; i < priv->auth_types->len; i++) {
 		if (priv->auth_types->pdata[i] == (gpointer)auth_class) {
 			g_ptr_array_remove_index (priv->auth_types, i);
-			g_type_class_unref (auth_class);
 			return TRUE;
 		}
 	}
@@ -194,7 +145,7 @@ remove_feature (SoupSessionFeature *feature, GType type)
 }
 
 static gboolean
-has_feature (SoupSessionFeature *feature, GType type)
+soup_auth_manager_has_feature (SoupSessionFeature *feature, GType type)
 {
 	SoupAuthManagerPrivate *priv = SOUP_AUTH_MANAGER_GET_PRIVATE (feature);
 	SoupAuthClass *auth_class;
@@ -219,7 +170,7 @@ soup_auth_manager_emit_authenticate (SoupAuthManager *manager, SoupMessage *msg,
 }
 
 static void
-attach (SoupSessionFeature *manager, SoupSession *session)
+soup_auth_manager_attach (SoupSessionFeature *manager, SoupSession *session)
 {
 	SoupAuthManagerPrivate *priv = SOUP_AUTH_MANAGER_GET_PRIVATE (manager);
 
@@ -402,6 +353,16 @@ get_auth_host_for_message (SoupAuthManagerPrivate *priv, SoupMessage *msg)
 	return host;
 }
 
+static void
+soup_auth_host_free (SoupAuthHost *host)
+{
+	g_clear_pointer (&host->auth_realms, soup_path_map_free);
+	g_clear_pointer (&host->auths, g_hash_table_destroy);
+
+	soup_uri_free (host->uri);
+	g_slice_free (SoupAuthHost, host);
+}
+
 static SoupAuth *
 lookup_auth (SoupAuthManagerPrivate *priv, SoupMessage *msg)
 {
@@ -575,8 +536,9 @@ requeue_if_proxy_authenticated (SoupMessage *msg, gpointer manager)
 }
 
 static void
-request_queued (SoupSessionFeature *manager, SoupSession *session,
-		SoupMessage *msg)
+soup_auth_manager_request_queued (SoupSessionFeature *manager,
+				  SoupSession *session,
+				  SoupMessage *msg)
 {
 	soup_message_add_status_code_handler (
 		msg, "got_headers", SOUP_STATUS_UNAUTHORIZED,
@@ -594,8 +556,10 @@ request_queued (SoupSessionFeature *manager, SoupSession *session,
 }
 
 static void
-request_started (SoupSessionFeature *feature, SoupSession *session,
-		 SoupMessage *msg, SoupSocket *socket)
+soup_auth_manager_request_started (SoupSessionFeature *feature,
+				   SoupSession *session,
+				   SoupMessage *msg,
+				   SoupSocket *socket)
 {
 	SoupAuthManager *manager = SOUP_AUTH_MANAGER (feature);
 	SoupAuthManagerPrivate *priv = SOUP_AUTH_MANAGER_GET_PRIVATE (manager);
@@ -613,9 +577,26 @@ request_started (SoupSessionFeature *feature, SoupSession *session,
 }
 
 static void
-request_unqueued (SoupSessionFeature *manager, SoupSession *session,
-		  SoupMessage *msg)
+soup_auth_manager_request_unqueued (SoupSessionFeature *manager,
+				    SoupSession *session,
+				    SoupMessage *msg)
 {
 	g_signal_handlers_disconnect_matched (msg, G_SIGNAL_MATCH_DATA,
 					      0, 0, NULL, NULL, manager);
 }
+
+static void
+soup_auth_manager_session_feature_init (SoupSessionFeatureInterface *feature_interface,
+					gpointer interface_data)
+{
+	soup_session_feature_default_interface =
+		g_type_default_interface_peek (SOUP_TYPE_SESSION_FEATURE);
+
+	feature_interface->attach = soup_auth_manager_attach;
+	feature_interface->request_queued = soup_auth_manager_request_queued;
+	feature_interface->request_started = soup_auth_manager_request_started;
+	feature_interface->request_unqueued = soup_auth_manager_request_unqueued;
+	feature_interface->add_feature = soup_auth_manager_add_feature;
+	feature_interface->remove_feature = soup_auth_manager_remove_feature;
+	feature_interface->has_feature = soup_auth_manager_has_feature;
+}
