@@ -3,8 +3,8 @@
  * Copyright (C) 2000-2003, Ximian, Inc.
  */
 
-#ifndef __SOUP_CONNECTION_H__
-#define __SOUP_CONNECTION_H__ 1
+#ifndef SOUP_CONNECTION_H
+#define SOUP_CONNECTION_H 1
 
 #include "soup-types.h"
 #include "soup-message-private.h"
@@ -35,31 +35,35 @@ typedef struct {
 GType soup_connection_get_type (void);
 
 
-#define SOUP_CONNECTION_REMOTE_URI        "remote-uri"
-#define SOUP_CONNECTION_SOCKET_PROPERTIES "socket-properties"
-#define SOUP_CONNECTION_STATE             "state"
-#define SOUP_CONNECTION_SSL               "ssl"
+typedef void  (*SoupConnectionCallback)        (SoupConnection   *conn,
+						guint             status,
+						gpointer          data);
 
-void            soup_connection_connect_async    (SoupConnection       *conn,
-						  GCancellable         *cancellable,
-						  GAsyncReadyCallback   callback,
-						  gpointer              user_data);
-gboolean        soup_connection_connect_finish   (SoupConnection       *conn,
-						  GAsyncResult         *result,
-						  GError              **error);
-gboolean        soup_connection_connect_sync     (SoupConnection       *conn,
-						  GCancellable         *cancellable,
-						  GError              **error);
-gboolean        soup_connection_start_ssl_sync   (SoupConnection       *conn,
-						  GCancellable         *cancellable,
-						  GError              **error);
-void            soup_connection_start_ssl_async  (SoupConnection       *conn,
-						  GCancellable         *cancellable,
-						  GAsyncReadyCallback   callback,
-						  gpointer              user_data);
-gboolean        soup_connection_start_ssl_finish (SoupConnection       *conn,
-						  GAsyncResult         *result,
-						  GError              **error);
+#define SOUP_CONNECTION_REMOTE_URI      "remote-uri"
+#define SOUP_CONNECTION_PROXY_RESOLVER  "proxy-resolver"
+#define SOUP_CONNECTION_SSL             "ssl"
+#define SOUP_CONNECTION_SSL_CREDENTIALS "ssl-creds"
+#define SOUP_CONNECTION_SSL_STRICT      "ssl-strict"
+#define SOUP_CONNECTION_SSL_FALLBACK    "ssl-fallback"
+#define SOUP_CONNECTION_ASYNC_CONTEXT   "async-context"
+#define SOUP_CONNECTION_USE_THREAD_CONTEXT "use-thread-context"
+#define SOUP_CONNECTION_TIMEOUT         "timeout"
+#define SOUP_CONNECTION_IDLE_TIMEOUT    "idle-timeout"
+#define SOUP_CONNECTION_STATE           "state"
+#define SOUP_CONNECTION_MESSAGE         "message"
+
+void            soup_connection_connect_async  (SoupConnection   *conn,
+						GCancellable     *cancellable,
+						SoupConnectionCallback callback,
+						gpointer          user_data);
+guint           soup_connection_connect_sync   (SoupConnection   *conn,
+						GCancellable     *cancellable);
+guint           soup_connection_start_ssl_sync   (SoupConnection   *conn,
+						  GCancellable     *cancellable);
+void            soup_connection_start_ssl_async  (SoupConnection   *conn,
+						  GCancellable     *cancellable,
+						  SoupConnectionCallback callback,
+						  gpointer          user_data);
 
 void            soup_connection_disconnect     (SoupConnection   *conn);
 
@@ -72,6 +76,7 @@ gboolean        soup_connection_is_tunnelled   (SoupConnection   *conn);
 SoupConnectionState soup_connection_get_state  (SoupConnection   *conn);
 void                soup_connection_set_state  (SoupConnection   *conn,
 						SoupConnectionState state);
+void                soup_connection_set_reusable (SoupConnection *conn);
 
 gboolean        soup_connection_get_ever_used  (SoupConnection   *conn);
 
@@ -80,6 +85,8 @@ void            soup_connection_send_request   (SoupConnection          *conn,
 						SoupMessageCompletionFn  completion_cb,
 						gpointer                 user_data);
 
+gboolean        soup_connection_get_ssl_fallback (SoupConnection   *conn);
+
 G_END_DECLS
 
-#endif /* __SOUP_CONNECTION_H__ */
+#endif /* SOUP_CONNECTION_H */
